==> capucine-0.2.tgz: capucine-0.2/ident.ml <==
(**************************************************************************)
(* Copyright (c) 2010, Romain BARDOU                                      *)
(* All rights reserved.                                                   *)
(*                                                                        *)
(* Redistribution and  use in  source and binary  forms, with  or without *)
(* modification, are permitted provided that the following conditions are *)
(* met:                                                                   *)
(*                                                                        *)
(* * Redistributions  of  source code  must  retain  the above  copyright *)
(*   notice, this list of conditions and the following disclaimer.        *)
(* * Redistributions in  binary form  must reproduce the  above copyright *)
(*   notice, this list of conditions  and the following disclaimer in the *)
(*   documentation and/or other materials provided with the distribution. *)
(* * Neither the  name of Capucine nor  the names of its contributors may *)
(*   be used  to endorse or  promote products derived  from this software *)
(*   without specific prior written permission.                           *)
(*                                                                        *)
(* THIS SOFTWARE  IS PROVIDED BY  THE COPYRIGHT HOLDERS  AND CONTRIBUTORS *)
(* "AS  IS" AND  ANY EXPRESS  OR IMPLIED  WARRANTIES, INCLUDING,  BUT NOT *)
(* LIMITED TO, THE IMPLIED  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR *)
(* A PARTICULAR PURPOSE  ARE DISCLAIMED. IN NO EVENT  SHALL THE COPYRIGHT *)
(* OWNER OR CONTRIBUTORS BE  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, *)
(* SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES (INCLUDING,  BUT  NOT *)
(* LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF USE, *)
(* DATA, OR PROFITS; OR BUSINESS  INTERRUPTION) HOWEVER CAUSED AND ON ANY *)
(* THEORY OF  LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY,  OR TORT *)
(* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING  IN ANY WAY OUT OF THE USE *)
(* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   *)
(**************************************************************************)

type t = {
  uid: int;
  name: string;
  uname: string;
  unifiable: bool;
}

(* To each [name], associate the number of identifiers which have been
   created using this same name.
   Thus, we are able to give to variables [x, y, x, z] the following
   unique names: [x, y, x0, z] or something similar. *)
let tbl: (string, int) Hashtbl.t = Hashtbl.create 169

let create =
  let c = ref (-1) in
  fun ?(unifiable = false) name ->
    let s =
      if !Options.output_kind = Options.Why3 then
        String.lowercase name
      else
        name
    in
    incr c;
    let nid = try Hashtbl.find tbl s + 1 with Not_found -> -1 in
    Hashtbl.replace tbl s nid;
    let uname = if nid = -1 then s else s ^ string_of_int nid in
    {
      uid = !c;
      name = name;
      uname = uname;
      unifiable = unifiable;
    }

let name x = x.name

let uname x = x.uname

let compare x y = compare x.uid y.uid

let unifiable x = x.unifiable

==> capucine-0.3.tgz: capucine-0.3/ident.ml <==
(**************************************************************************)
(* Copyright (c) 2010, Romain BARDOU                                      *)
(* All rights reserved.                                                   *)
(*                                                                        *)
(* Redistribution and  use in  source and binary  forms, with  or without *)
(* modification, are permitted provided that the following conditions are *)
(* met:                                                                   *)
(*                                                                        *)
(* * Redistributions  of  source code  must  retain  the above  copyright *)
(*   notice, this list of conditions and the following disclaimer.        *)
(* * Redistributions in  binary form  must reproduce the  above copyright *)
(*   notice, this list of conditions  and the following disclaimer in the *)
(*   documentation and/or other materials provided with the distribution. *)
(* * Neither the  name of Capucine nor  the names of its contributors may *)
(*   be used  to endorse or  promote products derived  from this software *)
(*   without specific prior written permission.                           *)
(*                                                                        *)
(* THIS SOFTWARE  IS PROVIDED BY  THE COPYRIGHT HOLDERS  AND CONTRIBUTORS *)
(* "AS  IS" AND  ANY EXPRESS  OR IMPLIED  WARRANTIES, INCLUDING,  BUT NOT *)
(* LIMITED TO, THE IMPLIED  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR *)
(* A PARTICULAR PURPOSE  ARE DISCLAIMED. IN NO EVENT  SHALL THE COPYRIGHT *)
(* OWNER OR CONTRIBUTORS BE  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, *)
(* SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES (INCLUDING,  BUT  NOT *)
(* LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF USE, *)
(* DATA, OR PROFITS; OR BUSINESS  INTERRUPTION) HOWEVER CAUSED AND ON ANY *)
(* THEORY OF  LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY,  OR TORT *)
(* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING  IN ANY WAY OUT OF THE USE *)
(* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   *)
(**************************************************************************)

type t = {
  uid: int;
  name: string;
  pname: string;
  uname: string;
}

(* To each [name], associate the number of identifiers which have been
   created using this same name.
   Thus, we are able to give to variables [x, y, x, z] the following
   unique names: [x, y, x0, z] or something similar. *)
let tbl: (string, int) Hashtbl.t = Hashtbl.create 169

let create =
  let c = ref (-1) in
  fun name ->
    let s =
      if !Options.output_kind = Options.Why3 then
        String.lowercase name
      else
        name
    in
    incr c;
    let nid = try Hashtbl.find tbl s + 1 with Not_found -> -1 in
    Hashtbl.replace tbl s nid;
    let uname = if nid = -1 then s else s ^ string_of_int nid in
    {
      uid = !c;
      name = name;
      pname = s;
      uname = uname;
    }

let name x = x.name

let uname x = x.uname

let compare x y = compare x.uid y.uid

let dummy =
  {
    uid = -1;
    name = "dummy";
    pname = "$dummy$";
    uname = "#dummy#";
  }

let pp fmt x =
  Format.fprintf fmt "%s" x.pname

let upp fmt x =
  Format.fprintf fmt "%s" x.uname
